/*
*******************************************************************************
* $Header: $
*
*  Copyright (c) 2018-2018 VIVOTEK Inc. All rights reserved.
*
*  +-----------------------------------------------------------------+
*  | THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY ONLY BE USED |
*  | AND COPIED IN ACCORDANCE WITH THE TERMS AND CONDITIONS OF SUCH  |
*  | A LICENSE AND WITH THE INCLUSION OF THE THIS COPY RIGHT NOTICE. |
*  | THIS SOFTWARE OR ANY OTHER COPIES OF THIS SOFTWARE MAY NOT BE   |
*  | PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON. THE   |
*  | OWNERSHIP AND TITLE OF THIS SOFTWARE IS NOT TRANSFERRED.        |
*  |                                                                 |
*  | THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT   |
*  | ANY PRIOR NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY |
*  | VIVOTEK INC.                                                    |
*  +-----------------------------------------------------------------+
*
* $History: $
*/

/*!
 *******************************************************************************
 * Copyright 2000-2018 VIVOTEK, Inc. All rights reserved.
 *
 * \file
 * PT5111.c
 *
 * \brief
 * Source file of motor control driver.
 *
 * \date
 * 2018/04/09
 *
 * \author
 * Tim Kuo <Tim.Kuo@vivotek.com>
 *
 *******************************************************************************
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/cdev.h>
#include <asm/io.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <asm/uaccess.h>

#include "icr_locals.h"


//-------------------------------------------------------------------------//
//channel
//bit 14~15
#define GROUP_A						0x0		// channel 1 and channel 2
#define GROUP_B						0x4000	// channel 3 and channel 4
#define GROUP_C						0x8000	// channel 5 and channel 7
//-------------------------------------------------------------------------//
//register
//bit 10~13
#define REG_PULSE_RATE_0			0xC00
#define REG_PULSE_RATE_RANGE		0x1000

#define REG_PULSE1					0x400
#define REG_PULSE2					0x800

#define REG_PRE_EXCITATION			0x2800
#define REG_POST_EXCITATION			0x2C00

#define REG_OPERATION_CTRL			0x3C00

#define REG_PWM_MODE				0x3000	// Only for Channel 5 and 7, Channels 5 and 7 are full-swing PWM
#define REG_PWM_DUTY_CH5			0x3400	// Only for Channel 5
//-------------------------------------------------------------------------//
//setting
//bit 9
#define SET_FORWARD					0x0
#define SET_BACKWARD				0x200

#define SET_OPERATION_DISABLE		0x0
#define SET_OPERATION_ENABLE		0x200

//bit 8
#define SET_PWM_STOP_MODE_OFF		0x0
#define SET_PWM_STOP_MODE_BRAKE		0x100

#define SET_OPERATION_STATE_RELEASE	0x0
#define SET_OPERATION_STATE_HOLD	0x100

//bit 8~9

#define	SET_DUTY_0					0x0
#define	SET_DUTY_100				0x300

//bit 7
#define SET_EXCITATION_DISABLE		0x0
#define SET_EXCITATION_ENABLE		0x80

#define SET_PWM_CTRL_MODE_BRAKE		0x0
#define SET_PWM_CTRL_MODE_REVERSE	0x80

//bit 6
#define SET_PWM_CARRIER_FREQ_D32	0x0
#define SET_PWM_CARRIER_FREQ_D48	0x40

#define SET_CH7_OPERATION_DISABLE	0x0
#define SET_CH7_OPERATION_ENABLE	0x40

//bit 6~8
#define SET_MICOROSTEP				0x0
#define SET_1_2PHASE_100			0x40	// 100%
#define SET_1_2PHASE_70				0x80	//  70%
#define SET_2_2PHASE				0xC0

#define SET_FULL_SWING_PWM			0x100	// Only for Channel 5 and 7, Channels 5 and 7 are full-swing PWM

//bit 6~9
#define SET_RANGE3_2				0x180	// 3.2 us	
#define SET_RANGE1_6				0x1C0	// 1.6 us	
#define SET_RANGE0_8				0x200	// 0.8 us	
#define SET_RANGE0_4				0x240	// 0.4 us	

//bit 6~9
#define SET_EXCITATION_TIME			0x20	// (4096*[2]/OSCIN[1Mhz]) 8.192ms

//bit 0~9
#define SET_PULSE_RATE_NUM_500		0x1F4	// 500
#define SET_PULSE_RATE_NUM_625		0x271	// 625
#define SET_PULSE_RATE_NUM_1000		0x3E8	// 1000

//-------------------------------------------------------------------------//
#ifndef __iomem
#define __iomem
#endif

unsigned int g_ulSPI4_CS = 0;

void pt5111_init_cfg(void)
{
	unsigned char framemode = 0;
	unsigned char spo = 1;
	unsigned char sph = 1;
	unsigned char datawidth = 16;

	unsigned char scr = 24;
	unsigned char cpsdvsr = 8;

	spi_disable();
	hi_ssp_set_spi4_reverse();
	hi_ssp_set_frameform(framemode, spo, sph, datawidth);
	hi_ssp_set_serialclock(scr, cpsdvsr);
	hi_ssp_alt_mode_set(1);
	hi_ssp_enable();

	spi_write_a16byte(0xCFC0, 0);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 1);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 2);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 3);//1100 1111 1100 0000   ps
}

int Set_SPI_CS_PIN(unsigned long ulSpics)
{
	g_ulSPI4_CS = (unsigned int)ulSpics;
	return S_OK;
}

void pt5111_SetDrivingMode(void)
{
	unsigned int SPI_cmd = 0x0;

	SPI_cmd |= GROUP_C | SET_FORWARD | SET_FULL_SWING_PWM;
	
	spi_write_a16byte(SPI_cmd, g_ulSPI4_CS);
}

void pt5111_SetPWMCtrlMode(void)
{
	unsigned int SPI_cmd = 0x0;

	SPI_cmd |= GROUP_C | REG_PWM_MODE | SET_BACKWARD | SET_PWM_STOP_MODE_BRAKE | SET_PWM_CTRL_MODE_REVERSE;
	
	spi_write_a16byte(SPI_cmd, g_ulSPI4_CS);
}

void pt5111_SetDuty0(void)
{
	unsigned int SPI_cmd = 0x0;

	SPI_cmd |= GROUP_C | REG_PWM_DUTY_CH5;
	
	spi_write_a16byte(SPI_cmd, g_ulSPI4_CS);
}

void pt5111_SetDuty100(void)
{
	unsigned int SPI_cmd = 0x0;

	SPI_cmd |= GROUP_C | REG_PWM_DUTY_CH5 | SET_DUTY_100;
	
	spi_write_a16byte(SPI_cmd, g_ulSPI4_CS);
}

int pt5111_SetOperation(bool bEnable)
{
	unsigned int SPI_cmd = GROUP_C;

	if (bEnable)
	{
		SPI_cmd |= REG_OPERATION_CTRL | SET_OPERATION_ENABLE | SET_OPERATION_STATE_RELEASE | SET_EXCITATION_DISABLE | SET_CH7_OPERATION_DISABLE;
	}
	else
	{
		SPI_cmd |= REG_OPERATION_CTRL | SET_OPERATION_DISABLE | SET_OPERATION_STATE_RELEASE | SET_EXCITATION_DISABLE | SET_CH7_OPERATION_DISABLE;
	}
	spi_write_a16byte(SPI_cmd, g_ulSPI4_CS);

	return S_OK;
}

void ICR_enable(void)
{
	hi_ssp_enable();
	pt5111_SetDrivingMode();
	pt5111_SetPWMCtrlMode();
	pt5111_SetDuty0();
	pt5111_SetOperation(TRUE);
	mdelay(IR_EXCITATION_TIME);
	pt5111_SetOperation(FALSE);
}


void ICR_disable(void)
{
	hi_ssp_enable();
	pt5111_SetDrivingMode();
	pt5111_SetPWMCtrlMode();
	pt5111_SetDuty100();
	pt5111_SetOperation(TRUE);
	mdelay(IR_EXCITATION_TIME);
	pt5111_SetOperation(FALSE);
}

INT hi_ssp_init(void)
{
#if 0
	reg_ssp_base_va = ioremap_nocache((unsigned int)SSP_BASE, (unsigned int)SSP_SIZE);
	if (!reg_ssp_base_va)
	{
		printk("Kernel: ioremap ssp base failed!\n");
	    return -ENOMEM;
	}

	reg_misc_base_va = ioremap_nocache((unsigned int)MISC_BASE, (unsigned int)MISC_SIZE);
	if (!reg_misc_base_va)
	{
		printk("Kernel: ioremap ssp base failed!\n");
	    return -ENOMEM;
	}

	//DBG(emDebugMsg, "hi_ssp_init ok!\n");
#endif
    return S_OK;
}

void hi_ssp_exit(void)
{
#if 0
    hi_ssp_disable();

    iounmap((void*)reg_ssp_base_va);
	iounmap((void*)reg_misc_base_va);
#endif
}

