#include <linux/init.h>
#include <linux/module.h>
#include <linux/cdev.h>
#include <asm/io.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <asm/uaccess.h>

#include "icr_hi3559.h"
#include "icr_locals.h"

static void icr_cleanup_module(void);

#define ICR_LAST_MODIFY_DATETIME	"Mon May. 28 19:30:00 CST 2018"
#define ICR_VERSION_STRING			"1.0.0.0"
volatile char icr_rcsid[] = "$Id: " ICR_VERSION_STRING " icr " ICR_LAST_MODIFY_DATETIME " $";

TICRLocalInfo	g_tICRDev;
static struct class *icrclass	= NULL;
static int	g_ICR_major			= ICR_MAJOR_DEFAULT; //ICR_MAJOR_DEFAULT;

static bool	CtrlLogic			= ICR_CTRL_LOGIC_DEFAULT;

static UINT	ICR_STATUS[4]		= {ICR_DAY};

extern unsigned int g_ulSPI4_CS;

module_param(g_ICR_major		, uint, S_IRUGO);
module_param(CtrlLogic		, bool, S_IRUGO);

int ICR_open(struct inode * inode, struct file * filp)
{
	mutex_lock(&g_tICRDev.mutex_lock);

	/* ICR dev has been opened */
	if (g_tICRDev.bOpenStatus == ICR_DEV_OPENED)
	{
		/* unlock spin lock */
		mutex_unlock(&g_tICRDev.mutex_lock);
		DBG("ICR dev has been opened.\n");
		return -EBUSY;
	}

	/* set ICR dev status to opened */
	g_tICRDev.bOpenStatus = ICR_DEV_OPENED;
	DBG("ICR module open success\n");
	
	spi_write_a16byte(0xCFC0, 0);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 1);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 2);//1100 1111 1100 0000   ps
	spi_write_a16byte(0xCFC0, 3);//1100 1111 1100 0000   ps

	mutex_unlock(&g_tICRDev.mutex_lock);

	return S_OK;
}

int ICR_release(struct inode * inode, struct file * filp)
{
	if (g_tICRDev.bOpenStatus == ICR_DEV_OPENED)
	{
		/* setting status of channel is not yet opened*/
		g_tICRDev.bOpenStatus = ICR_DEV_CLOSEED;

		//hi_ssp_disable();

		DBG("GPIO module close success\n");
	}

	return S_OK;
}

void ICR_switch_day_night(int iMode ,unsigned long ulChannel)
{
	iMode = g_tICRDev.bCtrlLogic ^ iMode;
	Set_SPI_CS_PIN(ulChannel);
	switch (iMode)
	{
		case DAY_MODE:
			ICR_disable();
			mdelay(IR_CUT_HW_DELAY);
			ICR_disable();
			break;

		case NIGHT_MODE:
			ICR_enable();
			mdelay(IR_CUT_HW_DELAY);
			ICR_enable();
			break;
	}
}

long ICR_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	mutex_lock(&g_tICRDev.mutex_lock);
	switch(cmd)
	{
		case ICR_DAY:
			ICR_switch_day_night(DAY_MODE, arg);
			printk("CH[%lu] ICR_DAY\n", arg);
			ICR_STATUS[g_ulSPI4_CS] = ICR_STATUS_DAY;
			break;

		case ICR_NIGHT:
			ICR_switch_day_night(NIGHT_MODE, arg);
			printk("CH[%lu] ICR_NIGHT\n", arg);
			ICR_STATUS[g_ulSPI4_CS] = ICR_STATUS_NIGHT;
			break;

		case ICR_GETSTATUS:
			put_user(((ICR_STATUS[g_ulSPI4_CS] == ICR_STATUS_NIGHT) ? ICR_STATUS_NIGHT : ICR_STATUS_DAY), (int __user *)arg);
			break;

		case ICR_DEVICE_SELECT:
			Set_SPI_CS_PIN(arg);
			break;

		default:
			printk(KERN_INFO "[ICR] ICR does not support ioctl flag %d\n", cmd);
			mutex_unlock(&g_tICRDev.mutex_lock);
			return -ENOTTY;
	}

	mutex_unlock(&g_tICRDev.mutex_lock);
	return S_OK;
}

static void ICR_exit(void)
{
	if (g_ICR_major != 0)
	{
		unregister_chrdev(g_ICR_major, "ICR");
	}

	printk(KERN_INFO "ICR exit. Goodbye.\n");

	icr_cleanup_module();
}

/* Device File Operations */
static struct file_operations ICR_fops = {
	owner:			THIS_MODULE,
	unlocked_ioctl:	ICR_ioctl,
	open:			ICR_open,
	release:		ICR_release,
};

static int ICR_init(void)
{
	struct class_device *device = NULL;
	UINT	uiDevminor = 0;
	int 	iRet = 0;

	printk("ICR Device Driver version %s\n", ICR_VERSION_STRING);

	g_tICRDev.bCtrlLogic						= CtrlLogic;
	g_tICRDev.bOpenStatus						= ICR_DEV_CLOSEED;

	/*****************************************************
	* Register the character driver.
	*****************************************************/
	iRet = register_chrdev(g_ICR_major, "ICR", &ICR_fops);
	if (iRet < 0)
	{
		DBG(" can't get major %d\n", g_ICR_major);
		goto fail_init;
	}

	// Create device class in sysfs
	if (!icrclass)
	{
		icrclass = class_create(THIS_MODULE, ICR_VFS_INTERFACE);
		if (IS_ERR(icrclass))
		{
			unregister_chrdev(g_ICR_major, ICR_VFS_INTERFACE);
			DBG("%s Class creation failure.\n", ICR_SYSLOG_ID);
			return PTR_ERR(icrclass);
		}
	}

	device = (struct class_device *)device_create(icrclass, NULL, MKDEV(g_ICR_major, uiDevminor), NULL, "icr");

	if (IS_ERR(device))
	{
		DBG("Device creation failure.\n");
		icr_cleanup_module();
		return PTR_ERR(device);
	}

	if (g_ICR_major == 0)
	{
		g_ICR_major = iRet; /* dynamic */
	}

	DBG("ICR Major Number = %d\n", g_ICR_major);
	DBG("make node with 'mknod /dev/icr\n");

	//hi_ssp_init();

	mutex_init(&g_tICRDev.mutex_lock);
	return S_OK; /* succeed */

fail_init:
	ICR_exit();
	return iRet;
}

/*!*************************************************************************
 * Cleanup icr module while rmmod.
 ****************************************************************************/
void __exit icr_cleanup_module ()
{
	device_destroy(icrclass, MKDEV(g_ICR_major, 0));
	DBG("Destroy class device: (%d, %d).\n", g_ICR_major, 0);


	class_destroy(icrclass);
	unregister_chrdev(g_ICR_major, ICR_VFS_INTERFACE);

	//hi_ssp_exit();

	printk("\n%s Bye...\n", ICR_SYSLOG_ID);
}

module_init(ICR_init);
module_exit(ICR_exit);

MODULE_AUTHOR("Tim.Kuo@vivotek.com");
MODULE_LICENSE("Dual BSD/GPL");

